var Est96000_DivergenciasInventarioCego_Controller = function(ZHPromise, metaDataFactory, templateManager, ScreenService, ZhLayoutPreferences, MessageService, General, EstUtil, ZhFilterPreferences, Est96000_DivergenciasInventarioCego_Services) {

    let IDUTILALMOX, IDUTILLCESTQ, IDUTILLOTE, DTPROCESSA, IDINVESTCEGO, IDOBREALACEDIV;

    // Open initial screen filter with default values
    this.onOpenFilter = function(widget) {
        let rowsToSet = {};
        Est96000_DivergenciasInventarioCego_Services.getFilialLogada().then(success => {
            rowsToSet.NMFILIAL = success.dataset.filial[0].NMFILIAL;
            rowsToSet.CDFILIAL = success.dataset.filial[0].CDFILIAL;

            Est96000_DivergenciasInventarioCego_Services.getParamFilialData().then(success => {
                const paramFilialData = success.dataset.paramfilial[0];

                DTPROCESSA = paramFilialData.DTPROCESSA;
                IDINVESTCEGO = paramFilialData.IDINVESTCEGO;
                IDOBREALACEDIV = paramFilialData.IDOBREALACEDIV;

                const dtInicial = DTPROCESSA;
                const dtFinal = moment().endOf('month').format('DD/MM/YYYY');

                rowsToSet.DTCONTAGEM = { "start": dtInicial, "end": dtFinal, "formattedStart": dtInicial, "formattedEnd": dtFinal };

                ZhFilterPreferences.openFilterWidget(widget, rowsToSet).then(success => {
                    templateManager.container.getActiveWidget('zhFilterPreferencesSelectConditions').getField("NMFILIAL").reload();
                });

            });
        });
    };

    // Creates parameter bag to send to backend
    this.setFilterParams = function(widget, preference = null) {
        const row = preference.FILTER_ROW;

        let startDate = row.DTCONTAGEM.start;
        let endDate = row.DTCONTAGEM.end;
        let CDFILIAL = row.CDFILIAL;

        let params = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": CDFILIAL
            },
            {
                "name": "IDSITESTCEGO",
                "operator": "=",
                "value": "AP"
            },
            {
                "name": "DTCONTAGEM",
                "operator": "BETWEEN",
                "value": [startDate, endDate]
            }
        ];

        widget.dataSourceFilter = params;
        widget.reload();
    };

    // This opens swipe details without change widget mode to view
    this.onOpenDetail = function(widget) {
        const InventoryTabbed = templateManager.container.getWidget('InventoryTabbed');
        const writeOffTabbed = templateManager.container.getWidget('writeOffItemGrid');
        const controlTabbed = templateManager.container.getWidget('controlItemGrid');

        ScreenService.openSwipe(InventoryTabbed).then(_ => {
            let promises = [];
            promises.push(writeOffTabbed.reload());
            promises.push(controlTabbed.reload());

            return ZHPromise.all(promises).then(_ => {
                this.actionsVisibility(writeOffTabbed);
            });
        });
    };

    // This function saves approved inventories or send them back to retyping situation 
    this.reportProcess = function(args, afterOpenReport = false) {
        const widget = args.owner;
        const descriptionField = widget.getField('DESCRIPTIONDATES');
        const almoxField = widget.getField('CDALMOXARIFE');
        const localField = widget.getField('CDLOCALESTOQ');

        if (afterOpenReport) {
            descriptionField.grouped = false;
            almoxField.grouped = false;
            localField.grouped = false;
            ZhLayoutPreferences.applyOriginalLayout(widget);

        }
        else {
            let writeOffLabel = "Relatório de Divergências do Estoque Cego - Itens a Baixar ";
            let controlLabel = "Relatório de Divergências do Estoque Cego - Itens a Controlar";

            args.e.metadata.title = widget.name === 'writeOffItemGrid' ? writeOffLabel : controlLabel;

            descriptionField.grouped = true;
            almoxField.grouped = true;
            localField.grouped = true;
            ZhLayoutPreferences.applyOriginalLayout(widget);
        }
    };

    // Manages actions functionalities according to buttons
    this.actionsManagementHandler = function(widget, action, firstTime = true) {

        widget = widget.name == 'writeOffItemGrid' ? widget : templateManager.container.getWidget('writeOffItemGrid');
        const rows = widget.dataSource.data;
        const ignoredKeys = ["__rightActions", "__leftActions", "__createdLocal", "__index", "$error"];
        const InventoryRow = widget.getParent().getParent().currentRow;

        switch (action) {
            case 'accept':

                if (rows.length > 0) {
                    MessageService.confirmMessageByCode('ACCEPT_CONFIRMATION').then(_ => {
                        Est96000_DivergenciasInventarioCego_Services.actionsManagementHandler(InventoryRow, rows, true, action).then(_ => {

                        }).catch(
                            error => {
                                if (error.data) ScreenService.showMessage(error.data.error);
                                else ScreenService.showMessage(error);
                            }
                        );
                    });
                }
                else {
                    MessageService.confirmMessageByCode('OPERATION_CONFIRMATION').then(_ => {
                        Est96000_DivergenciasInventarioCego_Services.actionsManagementHandler(InventoryRow, rows, false, action);

                    }).catch(
                        error => {
                            if (error.data) ScreenService.showMessage(error.data.error);
                            else ScreenService.showMessage(error);
                        }
                    );
                }

                break;
            case 'refuse':
                if (firstTime) {
                    MessageService.confirmMessageByCode('REFUSE_CONFIRMATION').then(_ => {
                        const refusePopUp = templateManager.container.getWidget('refuseInventoryPopUp');

                        ScreenService.openPopup(refusePopUp).then(_ => {
                            refusePopUp.currentRow = Util.clone(InventoryRow, ignoredKeys);
                            refusePopUp.currentRow.NMFILIAL = EstUtil.getFieldValue('NMFILIAL', 'inventoryGrid', true);
                            refusePopUp.currentRow.DSOBSRECAJUST = '';
                            refusePopUp.getField('NMFILIAL').reload();
                        });

                    });
                }
                else {
                    const refuseWidget = templateManager.container.getWidget('refuseInventoryPopUp');

                    Est96000_DivergenciasInventarioCego_Services.actionsManagementHandler(refuseWidget.currentRow, [], '', action).then(success => {
                        MessageService.successNotificationByCode('REFUSE_SUCCESS');
                    }).catch(
                        error => {
                            if (error.data) ScreenService.showMessage(error.data.error);
                            else ScreenService.showMessage(error);
                        }
                    );
                }
                break;

            case 'adjust':
                MessageService.confirmMessageByCode('ADJUST_CONFIRMATION').then(_ => {
                    Est96000_DivergenciasInventarioCego_Services.actionsManagementHandler(InventoryRow, rows, false, action).then(success => {
                        let IrregularItems = success.dataset.ResultData.ItemsAjusted;
                        let ItemsWithoutCost = success.dataset.ResultData.ItemsWithoutCost;
                        let Nrlancestq = success.dataset.ResultData.nrlancestq;

                        if (IrregularItems.length == 0 && ItemsWithoutCost.length == 0)
                            MessageService.successNotificationByCode('GERA_AJUST_SUCCESS', 3000, { "NRLANCEST": Nrlancestq });

                        else {
                            if (IrregularItems.length != 0)
                                MessageService.showMessageByCode('WARNING_GERA_AJUST_IRREGULAR', { "NRLANCEST": Nrlancestq });

                            if (ItemsWithoutCost.length != 0) {
                                let messageItems = '<br>';
                                ItemsWithoutCost.forEach(item => {
                                    messageItems += '<br>' + item.CDPRODUTO + ' - ' + item.NMPRODUTO + '<br>';
                                });
                                MessageService.showMessageByCode('WARNING_GERA_AJUST_NO_COST_STOCK', { "ITEMS": messageItems });
                            }
                        }
                    }).catch(
                        error => {
                            if (error.data) ScreenService.showMessage(error.data.error);
                            else ScreenService.showMessage(error);
                        }
                    );
                });
                break;
            default:
                break;

        }
    };

    // Controls actions visibility on enter widget
    this.actionsVisibility = function(widget) {
        const InventoryRow = widget.getParent().getParent().currentRow;
        const controlTabbed = templateManager.container.getWidget('controlItemGrid');

        Est96000_DivergenciasInventarioCego_Services.getGroupOperatorInfo().then((success) => {
            
            const dataReturn = success.dataset.autgrupop_grupoper_opergrupop;
            const ValidateGroupOperator = (OperatorGroupInfo, action) => {
                if (action == 'adjust') {
                    return OperatorGroupInfo.filter((item) => {
                        return item.IDAUTGRUPOP === 'GAUD';
                    });
                }
                else {
                    return OperatorGroupInfo.filter((item) => {
                        return item.IDAUTGRUPOP === 'GGER' || item.IDAUTGRUPOP === 'GAUD';
                    });
                }
            };
            const DtprocessaDate = Est96000_DivergenciasInventarioCego_Services.getDate(DTPROCESSA);
            const DtContagemDate = Est96000_DivergenciasInventarioCego_Services.getDate(InventoryRow.DTCONTAGEM);
            const DtutilDate = Est96000_DivergenciasInventarioCego_Services.getDate(InventoryRow.DTDIAUTIL);
            let hasItems = (widget.dataSource.data.length != 0 || controlTabbed.dataSource.data.length != 0) ? true : false;

            if (
                ValidateGroupOperator(dataReturn, 'accept').length != 0 &&
                InventoryRow.IDSITUACEITEINV == "P" && hasItems &&
                (DtprocessaDate > DtContagemDate &&
                    DtprocessaDate <= DtutilDate)
            ) {
                General.setEnableAction('accept', widget.name);
                General.setEnableAction('refuse', widget.name);
                if (IDOBREALACEDIV == 'S')
                    General.setDisableAction('refuse', widget.name);
            }
            else {
                General.setDisableAction('accept', widget.name);
                General.setDisableAction('refuse', widget.name);
            }
            if (
                ValidateGroupOperator(dataReturn, 'adjust').length != 0 &&
                    hasItems && (IDINVESTCEGO == "S" &&
                    InventoryRow.IDINVGERAAJUSTE != "S" &&
                    InventoryRow.IDSITUACEITEINV != "R") &&
                    (DtprocessaDate > DtContagemDate &&
                    DtprocessaDate <= DtutilDate)
            ) {
                General.setEnableAction('adjust', widget.name);
            }
            else {
                General.setDisableAction('adjust', widget.name);
            }
        });
    };

    //this function closes the refuse inventory pop up
    this.onClosePopUpRefuseInventory = function(widget) {
        const writeOffGrid = templateManager.container.getWidget('writeOffItemGrid');
        ScreenService.closePopup();
        writeOffGrid.activate();
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est96000_DivergenciasInventarioCego_Controller', Est96000_DivergenciasInventarioCego_Controller);
});
